@@ -25,9 +25,9 @@ import fixtures
 from oslo_utils import timeutils
 import prettytable
 import six
-from tempest_lib import base
-from tempest_lib.common import ssh as connection
-from tempest_lib import exceptions as exc
+from tempest.lib import base
+from tempest.lib.common import ssh as connection
+from tempest.lib import exceptions as exc
 
 from sahara_tests.scenario import clients
 from sahara_tests.scenario import timeouts
@@ -41,6 +41,7 @@ CHECK_OK_STATUS = "OK"
 CHECK_FAILED_STATUS = "FAILED"
 CLUSTER_STATUS_ACTIVE = "Active"
 CLUSTER_STATUS_ERROR = "Error"
+HEALTH_CHECKS = ["RED", "YELLOW", "GREEN"]
 
 
 def track_result(check_name, exit_with_error=True):
@@ -88,7 +89,7 @@ class BaseTestCase(base.BaseTestCase):
         self._init_clients()
         timeouts.Defaults.init_defaults(self.testcase)
         self.testcase['ssh_username'] = self.sahara.register_image(
-            self.nova.get_image_id(self.testcase['image']),
+            self.glance.get_image_id(self.testcase['image']),
             self.testcase).username
         self.key = self.testcase.get('key_name')
         if self.key is None:
@@ -97,10 +98,11 @@ class BaseTestCase(base.BaseTestCase):
         # save the private key if retain_resources is specified
         # (useful for debugging purposes)
         if self.testcase['retain_resources'] or self.key is None:
-            private_key_file = os.path.join(self.results_dir, self.key_name +
-                                            '.key')
-            with open(private_key_file, 'a') as private_key_file:
+            private_key_file_name = os.path.join(self.results_dir,
+                                                 self.key_name + '.key')
+            with open(private_key_file_name, 'w+') as private_key_file:
                 private_key_file.write(self.private_key)
+            os.chmod(private_key_file_name, 0o600)
         self.plugin_opts = {
             'plugin_name': self.testcase['plugin_name'],
             'hadoop_version': self.testcase['plugin_version']
@@ -137,6 +139,7 @@ class BaseTestCase(base.BaseTestCase):
             insecure=not self.credentials.get('ssl_verify', False),
             cacert=self.credentials.get('ssl_cert'),
             tenant_name=tenant_name)
+        self.glance = clients.GlanceClient(session=session)
 
     def create_cluster(self):
         self.cluster_id = self.sahara.get_cluster_id(
@@ -152,9 +155,8 @@ class BaseTestCase(base.BaseTestCase):
         cluster = self.sahara.get_cluster(self.cluster_id, show_progress=True)
         self._get_proxy(cluster)
         self.check_cinder()
-        if not getattr(cluster, "provision_progress", None):
-            return
-        self._check_event_logs(cluster)
+        if self.check_feature_available("provision_progress"):
+            self._check_event_logs(cluster)
 
     def _get_proxy(self, cluster):
         for ng in cluster.node_groups:
@@ -216,6 +218,7 @@ class BaseTestCase(base.BaseTestCase):
                 self._job_batching(pre_exec)
                 pre_exec = []
                 batching = batching_size
+        self.check_verification(self.cluster_id)
 
     def _job_batching(self, pre_exec):
         job_exec_ids = []
@@ -233,7 +236,8 @@ class BaseTestCase(base.BaseTestCase):
                 url = self._create_swift_data(source, destination)
             if ds['type'] == 'hdfs':
                 url = self._create_dfs_data(source, destination,
-                                            ds.get('hdfs_username', 'oozie'),
+                                            self.testcase.get('hdfs_username',
+                                                              'hadoop'),
                                             ds['type'])
             if ds['type'] == 'maprfs':
                 url = self._create_dfs_data(source, destination,
@@ -381,7 +385,10 @@ class BaseTestCase(base.BaseTestCase):
                             'maprfs': 'hadoop fs'}
 
         hdfs_dir = utils.rand_name("/user/%s/data" % hdfs_username)
-        inst_ip = self._get_nodes_with_process()[0]["management_ip"]
+        instances = self._get_nodes_with_process('namenode')
+        if len(instances) == 0:
+            instances = self._get_nodes_with_process('CLDB')
+        inst_ip = instances[0]["management_ip"]
         self._run_command_on_node(
             inst_ip,
             "sudo su - -c \"%(prefix)s -mkdir -p %(path)s \" %(user)s" % {
@@ -508,6 +515,9 @@ class BaseTestCase(base.BaseTestCase):
     def _create_node_group_templates(self):
         ng_id_map = {}
         floating_ip_pool = None
+        security_group = None
+        proxy_exist = False
+
         if self.network['type'] == 'neutron':
             floating_ip_pool = self.neutron.get_network_id(
                 self.network['public_network'])
@@ -517,6 +527,8 @@ class BaseTestCase(base.BaseTestCase):
         node_groups = []
         for ng in self.testcase['node_group_templates']:
             node_groups.append(ng)
+            if ng.get('is_proxy_gateway', False):
+                proxy_exist = True
 
         for ng in node_groups:
             kwargs = dict(ng)
@@ -524,7 +536,16 @@ class BaseTestCase(base.BaseTestCase):
             kwargs['flavor_id'] = self._get_flavor_id(kwargs['flavor'])
             del kwargs['flavor']
             kwargs['name'] = utils.rand_name(kwargs['name'])
-            kwargs['floating_ip_pool'] = floating_ip_pool
+            if (not proxy_exist) or (proxy_exist and kwargs.get(
+                    'is_proxy_gateway', False)):
+                kwargs['floating_ip_pool'] = floating_ip_pool
+            if not kwargs.get('auto_security_group', True):
+                if security_group is None:
+                    sg_name = utils.rand_name('scenario')
+                    security_group = self.__create_security_group(sg_name)
+                    self.neutron.add_security_group_rule_for_neutron(
+                        security_group)
+                kwargs['security_groups'] = [security_group]
             ng_id = self.__create_node_group_template(**kwargs)
             ng_id_map[ng['name']] = ng_id
         return ng_id_map
@@ -596,7 +617,7 @@ class BaseTestCase(base.BaseTestCase):
         kwargs.update(self.plugin_opts)
         kwargs['name'] = utils.rand_name(kwargs.get('name', 'test'))
         kwargs['cluster_template_id'] = cluster_template_id
-        kwargs['default_image_id'] = self.nova.get_image_id(
+        kwargs['default_image_id'] = self.glance.get_image_id(
             self.testcase['image'])
         kwargs['user_keypair_id'] = self.key_name
 
@@ -638,13 +659,51 @@ class BaseTestCase(base.BaseTestCase):
         return ssh_session.exec_command(command)
 
     def _get_nodes_with_process(self, process=None):
+        if process is not None:
+            process = process.lower()
         nodegroups = self.sahara.get_cluster(self.cluster_id).node_groups
         nodes_with_process = []
         for nodegroup in nodegroups:
-            if not process or process in nodegroup['node_processes']:
-                nodes_with_process.extend(nodegroup['instances'])
+            for node_process in nodegroup['node_processes']:
+                if not process or process in node_process.lower():
+                    nodes_with_process.extend(nodegroup['instances'])
         return nodes_with_process
 
+    def _get_health_status(self, cluster):
+        try:
+            return cluster.verification['status']
+        except (AttributeError, KeyError):
+            return 'UNKNOWN'
+
+    def _poll_verification_status(self, cluster_id):
+        with fixtures.Timeout(
+                timeouts.Defaults.instance.timeout_poll_cluster_status,
+                gentle=True):
+            while True:
+                cluster = self.sahara.get_cluster(cluster_id)
+                status = self._get_health_status(cluster)
+                if status == 'UNKNOWN':
+                    print("Cluster verification did not start")
+                    break
+                if status in HEALTH_CHECKS:
+                    break
+                time.sleep(3)
+
+    @track_result("Check cluster verification")
+    def check_verification(self, cluster_id):
+        if self.check_feature_available("verification"):
+            self._poll_cluster_status(cluster_id)
+
+            # need to check if previous verification check is not
+            # in the status CHECKING
+            self._poll_verification_status(cluster_id)
+            self.sahara.start_cluster_verification(cluster_id)
+
+            # check if this verification check finished without errors
+            self._poll_verification_status(cluster_id)
+        else:
+            print("All tests for cluster verification were skipped")
+
     # client ops
 
     def __create_node_group_template(self, *args, **kwargs):
@@ -653,6 +712,12 @@ class BaseTestCase(base.BaseTestCase):
             self.addCleanup(self.sahara.delete_node_group_template, id)
         return id
 
+    def __create_security_group(self, sg_name):
+        id = self.neutron.create_security_group_for_neutron(sg_name)
+        if not self.testcase['retain_resources']:
+            self.addCleanup(self.neutron.delete_security_group_for_neutron, id)
+        return id
+
     def __create_cluster_template(self, *args, **kwargs):
         id = self.sahara.create_cluster_template(*args, **kwargs)
         if not self.testcase['retain_resources']:
@@ -716,6 +781,12 @@ class BaseTestCase(base.BaseTestCase):
             self.addCleanup(self.nova.delete_keypair, key)
         return key
 
+    def check_feature_available(self, feature_name):
+        if not getattr(self.sahara.get_cluster(self.cluster_id),
+                       feature_name, None):
+            return False
+        return True
+
     def tearDown(self):
         tbs = []
         table = prettytable.PrettyTable(["Check", "Status", "Duration, s"])
