@@ -17,6 +17,7 @@ from __future__ import print_function
 import time
 
 import fixtures
+from glanceclient import client as glance_client
 from keystoneauth1.identity import v3 as identity_v3
 from keystoneauth1 import session
 from neutronclient.neutron import client as neutron_client
@@ -27,8 +28,9 @@ from saharaclient.api import base as saharaclient_base
 from saharaclient import client as sahara_client
 from swiftclient import client as swift_client
 from swiftclient import exceptions as swift_exc
-from tempest_lib import exceptions as exc
+from tempest.lib import exceptions as exc
 
+from sahara_tests.scenario import timeouts
 from sahara_tests.scenario import utils
 
 
@@ -42,8 +44,6 @@ def get_session(auth_url=None, username=None, password=None,
                                 project_domain_name='default')
     return session.Session(auth=auth, verify=verify, cert=cert)
 
-from sahara_tests.scenario import timeouts
-
 
 class Client(object):
     def is_resource_deleted(self, method, *args, **kwargs):
@@ -93,6 +93,10 @@ class SaharaClient(Client):
     def scale_cluster(self, cluster_id, body):
         return self.sahara_client.clusters.scale(cluster_id, body)
 
+    def start_cluster_verification(self, cluster_id):
+        return self.sahara_client.clusters.verification_update(cluster_id,
+                                                               'START')
+
     def create_datasource(self, *args, **kwargs):
         data = self.sahara_client.data_sources.create(*args, **kwargs)
         return data.id
@@ -200,15 +204,6 @@ class NovaClient(Client):
     def __init__(self, *args, **kwargs):
         self.nova_client = nova_client.Client('2', *args, **kwargs)
 
-    def get_image_id(self, image_name):
-        if uuidutils.is_uuid_like(image_name):
-            return image_name
-        for image in self.nova_client.images.list():
-            if image.name == image_name:
-                return image.id
-
-        raise exc.NotFound(image_name)
-
     def get_flavor_id(self, flavor_name):
         if uuidutils.is_uuid_like(flavor_name) or flavor_name.isdigit():
             return flavor_name
@@ -257,6 +252,53 @@ class NeutronClient(Client):
             raise exc.NotFound(network_name)
         return networks[0]['id']
 
+    def create_security_group_for_neutron(self, sg_name):
+        security_group = self.neutron_client.create_security_group({
+            "security_group":
+                {
+                    "name": sg_name,
+                    "description": "Just for test"
+                }
+        })
+        return security_group['security_group']['id']
+
+    def get_security_group_id(self, sg_name):
+        for sg in (self.neutron_client.list_security_groups()
+                   ["security_groups"]):
+            if sg['name'] == sg_name:
+                return sg['id']
+
+        raise exc.NotFound(sg_name)
+
+    def add_security_group_rule_for_neutron(self, sg_id):
+        return self.neutron_client.create_security_group_rule({
+            "security_group_rules": [
+                {
+                    "direction": "ingress",
+                    "ethertype": "IPv4",
+                    "port_range_max": 65535,
+                    "port_range_min": 1,
+                    "protocol": "TCP",
+                    "remote_group_id": None,
+                    "remote_ip_prefix": None,
+                    "security_group_id": sg_id
+                },
+                {
+                    "direction": "egress",
+                    "ethertype": "IPv4",
+                    "port_range_max": 65535,
+                    "port_range_min": 1,
+                    "protocol": "TCP",
+                    "remote_group_id": None,
+                    "remote_ip_prefix": None,
+                    "security_group_id": sg_id
+                }
+            ]
+        })
+
+    def delete_security_group_for_neutron(self, sg_id):
+        return self.neutron_client.delete_security_group(sg_id)
+
 
 class SwiftClient(Client):
     def __init__(self, *args, **kwargs):
@@ -295,3 +337,16 @@ class SwiftClient(Client):
             return ex.http_status == 404
 
         return False
+
+
+class GlanceClient(Client):
+    def __init__(self, *args, **kwargs):
+        self.glance_client = glance_client.Client('2', *args, **kwargs)
+
+    def get_image_id(self, image_name):
+        if uuidutils.is_uuid_like(image_name):
+            return image_name
+        for image in self.glance_client.images.list():
+            if image.name == image_name:
+                return image.id
+        raise exc.NotFound(image_name)
