@@ -13,12 +13,15 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
+import os
 import sys
 
 from jsonschema import exceptions
+import pkg_resources as pkg
 import mock
 import testtools
 
+from sahara_tests import version
 from sahara_tests.scenario import runner
 
 
@@ -49,12 +52,8 @@ class RunnerUnitTest(testtools.TestCase):
 
         expected_default_credential = {
             "credentials": {
-                "os_username": "admin",
-                "os_auth_url": "http://localhost:5000/v2.0",
                 "sahara_url": None,
                 "sahara_service_type": "data-processing",
-                "os_password": "nova",
-                "os_tenant": "admin",
                 "ssl_cert": None,
                 "ssl_verify": False
             }
@@ -311,18 +310,19 @@ class RunnerUnitTest(testtools.TestCase):
     def test_runner_from_args(self, mock_sys, mock_sub, mock_validate):
         sys.argv = ['sahara_tests/scenario/runner.py',
                     'sahara_tests/unit/scenario/vanilla2_7_1.yaml.mako',
-                    '--args', 'OS_USERNAME:demo', 'OS_PASSWORD:demopwd',
-                    'OS_TENANT_NAME:demo', 'network_type:neutron',
+                    '--os-username', 'demo', '--os-password', 'demopwd',
+                    '--os-project-name', 'demo', '--os-auth-url', 'localhost',
+                    '--args', 'network_type:neutron',
                     'network_private_name:private',
                     'network_public_name:public',
                     'vanilla_two_six_image:hadoop_2_6_latest',
-                    'ci_flavor_id:2', 'OS_AUTH_URL:localhost']
+                    'ci_flavor_id:2']
         runner.main()
         expected = {
             'os_username': 'demo',
             'os_password': 'demopwd',
             'os_tenant': 'demo',
-            'os_auth_url': 'localhost'
+            'os_auth_url': 'localhost/v2.0'
         }
         self.assertTrue(self._isDictContainSubset(
             expected, mock_validate.call_args_list[0][0][0]['credentials']))
@@ -349,7 +349,10 @@ class RunnerUnitTest(testtools.TestCase):
         sys.argv = ['sahara_tests/scenario/runner.py',
                     '-V',
                     'sahara_tests/unit/scenario/templatevars_complete.ini',
-                    '-p', 'transient']
+                    '-p', 'transient', '--os-username', 'demo',
+                    '--os-password', 'demopwd',
+                    '--os-project-name', 'demo',
+                    '--os-auth-url', 'http://127.0.0.1:5000/v2']
         runner.main()
 
     @mock.patch('sahara_tests.scenario.validation.validate')
@@ -361,7 +364,10 @@ class RunnerUnitTest(testtools.TestCase):
         sys.argv = ['sahara_tests/scenario/runner.py',
                     '-V',
                     'sahara_tests/unit/scenario/templatevars_complete.ini',
-                    '-p', 'vanilla']
+                    '-p', 'vanilla', '--os-username', 'demo',
+                    '--os-password', 'demopwd',
+                    '--os-project-name', 'demo',
+                    '--os-auth-url', 'http://127.0.0.1:5000/v2']
         with testtools.ExpectedException(ValueError):
             runner.main()
 
@@ -373,12 +379,15 @@ class RunnerUnitTest(testtools.TestCase):
         sys.argv = ['sahara_tests/scenario/runner.py',
                     '-V',
                     'sahara_tests/unit/scenario/templatevars_complete.ini',
-                    '-p', 'spark', '-v', '1.0.0', '-r', 'kilo']
+                    '-p', 'spark', '-v', '1.3.1', '-r', 'liberty',
+                    '--os-username', 'demo', '--os-password', 'demopwd',
+                    '--os-project-name', 'demo',
+                    '--os-auth-url', 'http://127.0.0.1:5000/v2']
         runner.main()
         self.assertEqual('spark',
                          mock_validate.call_args[0][0]['clusters'][0][
                              'plugin_name'])
-        self.assertEqual('1.0.0',
+        self.assertEqual('1.3.1',
                          mock_validate.call_args[0][0]['clusters'][0][
                              'plugin_version'])
 
@@ -389,5 +398,87 @@ class RunnerUnitTest(testtools.TestCase):
         sys.argv = ['sahara_tests/scenario/runner.py',
                     '-V',
                     'sahara_tests/unit/scenario/templatevars_complete.ini',
-                    '-p', 'spark', '-v', '1.0.0', '--count', '4']
+                    '-p', 'spark', '-v', '1.3.1', '--release', 'liberty',
+                    '--count', '4',
+                    '--os-username', 'demo', '--os-password', 'demopwd',
+                    '--os-project-name', 'demo',
+                    '--os-auth-url', 'http://127.0.0.1:5000/v2']
         runner.main()
+
+    @mock.patch('subprocess.Popen',
+                return_value=_create_subprocess_communicate_mock())
+    @mock.patch('sys.exit', return_value=None)
+    def test_run_dir(self, mock_sys, mock_sub):
+        sys.argv = ['sahara_tests/scenario/runner.py',
+                    '-V',
+                    'sahara_tests/unit/scenario/templatevars_complete.ini',
+                    'sahara_tests/scenario/defaults/liberty',
+                    'sahara_tests/scenario/defaults/edp.yaml.mako',
+                    '--os-username', 'demo', '--os-password', 'demopwd',
+                    '--os-project-name', 'demo',
+                    '--os-auth-url', 'http://127.0.0.1:5000/v2', '--args',
+                    'ambari_2_1_image:ambari', 'fake_plugin_image:fake',
+                    'mapr_500mrv2_image:mapr', 'spark_1_3_image:spark',
+                    'hdp_two_image:hdp']
+        runner.main()
+
+    @mock.patch('sahara_tests.scenario.validation.validate')
+    @mock.patch('subprocess.Popen',
+                return_value=_create_subprocess_communicate_mock())
+    @mock.patch('sys.exit', return_value=None)
+    def test_credentials_envvars(self, mock_sys, mock_sub, mock_validate):
+        username = os.environ.get('OS_USERNAME', '')
+        password = os.environ.get('OS_PASSWORD', '')
+        auth_url = os.environ.get('OS_AUTHURL', '')
+        project_name = os.environ.get('OS_PROJECT_NAME', '')
+        os.environ['OS_USERNAME'] = 'demo_env'
+        os.environ['OS_PASSWORD'] = 'demopwd_env'
+        os.environ['OS_AUTH_URL'] = 'http://localhost:5000/v2.0'
+        os.environ['OS_PROJECT_NAME'] = 'project_env'
+        sys.argv = ['sahara_tests/scenario/runner.py',
+                    '-V',
+                    'sahara_tests/unit/scenario/templatevars_complete.ini',
+                    '-p', 'spark', '-v', '1.3.1', '--release', 'liberty',
+                    '--count', '4']
+        runner.main()
+        expected = {
+            'os_username': 'demo_env',
+            'os_password': 'demopwd_env',
+            'os_tenant': 'project_env',
+            'os_auth_url': 'http://localhost:5000/v2.0'
+        }
+        self.assertTrue(self._isDictContainSubset(
+            expected, mock_validate.call_args_list[0][0][0]['credentials']))
+
+        os.environ['OS_USERNAME'] = username
+        os.environ['OS_PASSWORD'] = password
+        os.environ['OS_AUTHURL'] = auth_url
+        os.environ['OS_PROJECT_NAME'] = project_name
+
+    @mock.patch('sahara_tests.scenario.validation.validate')
+    @mock.patch('subprocess.Popen',
+                return_value=_create_subprocess_communicate_mock())
+    @mock.patch('sys.exit', return_value=None)
+    def test_credentials_clouds(self, mock_sys, mock_sub, mock_validate):
+        unit_dir = pkg.resource_filename(version.version_info.package,
+                                         'unit')
+        os_client_config_file = os.environ.get('OS_CLIENT_CONFIG_FILE', '')
+        os.environ['OS_CLIENT_CONFIG_FILE'] = os.path.join(unit_dir,
+                                                           'scenario',
+                                                           'clouds.yaml')
+        sys.argv = ['sahara_tests/scenario/runner.py',
+                    '-V',
+                    'sahara_tests/unit/scenario/templatevars_complete.ini',
+                    '-p', 'spark', '-v', '1.0.0', '--count', '4',
+                    '--os-cloud', 'scenario_test']
+        runner.main()
+        expected = {
+            'os_username': 'demo_cloud',
+            'os_password': 'demo_cloud_pwd',
+            'os_tenant': 'demo_cloud_project',
+            'os_auth_url': 'http://localhost:5000/v2.0'
+        }
+        self.assertTrue(self._isDictContainSubset(
+            expected, mock_validate.call_args_list[0][0][0]['credentials']))
+        os.environ['OS_CLIENT_CONFIG_FILE'] = os_client_config_file
+
